<?php

class EntityReport {

  private $entity_name;
  private $bundle_name;
  private $entity;
  private $bundle;

  private $plural_label;

  public function __construct($entity_name, $bundle_name) {
    $this->entity_name = $entity_name;
    $this->bundle_name = $bundle_name;
    $this->entity = entity_get_info($entity_name);
    $this->bundle = $this->entity['bundles'][$bundle_name];
    $this->plural_label = $this->entity['plural label'];
  }

  public function getTotalCount() {
    $q = $this->getEntityQuery();
    $q->count();
    return $q->execute();
  }

  public function getTotalPublishedCount() {
    $q = $this->getEntityQuery();
    $q->propertyCondition('status', 1);
    $q->count();
    return $q->execute();
  }

  public function getFieldTypeLabel($field_name) {
    $field_types = field_info_field_types();
    $info = field_info_field($field_name);
    $instance = $this->getFieldInformation()[$field_name];

    $label = $field_types[$info['type']]['label'];

    switch ($info['type']) {
      case 'text_long':
      case 'text_with_summary':
        if (empty($instance['settings']['text_processing'])) {
          $label .= ' (Plain)';
        }
        else {
          $label .= ' (Filtered)';
        }
        break;

      case 'entityreference':
        if (!empty($view = $info['settings']['handler_settings']['view'])) {
          $label .= " (view.{$view['view_name']}.{$view['display_name']})";
        }
        else {
          $allowed_bundles = $info['settings']['handler_settings']['target_bundles'];
          $label .= ' (' . implode(', ', $allowed_bundles) . ')';
        }
        break;

      case 'taxonomy_term_reference':
        $allowed_vids = array_column($info['settings']['allowed_values'], 'vocabulary');
        $label .= ' (' . implode(', ', $allowed_vids) . ')';
        break;
    }

    return $label;
  }

  public function getFieldInformation() {
    $info = field_info_instances($this->entity_name, $this->bundle_name);
    return $info;
  }

  public function isTranslatableLabel() {
    $translation = $this->isTranslatable();
    switch ($translation) {
      case ENTITY_TRANSLATION_ENABLED:
        return 'Yes, with field translation';
      case TRANSLATION_ENABLED:
        return 'Yes, with node translation';
    }
  }

  public function isTranslatable() {
    switch ($this->entity_name) {
      case 'taxonomy_term':
        break;
      case 'node':
        $ret = variable_get('language_content_type_' . $this->bundle_name, 0);
    }
    return $ret;
  }

  /**
   * @return \EntityFieldQuery
   */
  private function getEntityQuery() {
    $q = new EntityFieldQuery();
    $q->entityCondition('entity_type', $this->entity_name);
    switch ($this->entity_name) {
      case 'taxonomy_term':
        $voc = taxonomy_vocabulary_machine_name_load($this->bundle_name);
        $q->propertyCondition('vid', $voc->vid);
        break;
      case 'node':
        $q->propertyCondition('type', $this->bundle_name);
        break;
    }
    return $q;
  }

  public function getPluralLabel() {
    if (!empty($this->plural_label)) {
      return $this->plural_label;
    }
    return '';
  }

  public function getFieldValueExample($field_name, $translation = FALSE) {
    $info = field_info_field($field_name);
    $cols = array_keys($info['columns']);
    $column = reset($cols);
    $table = $this->entity['base table'];
    $id = $this->entity['entity keys']['id'];
    $bundle = $this->entity['entity keys']['bundle'];

    $q = db_select("field_data_$field_name", 'f');
    $q->fields('f', ['entity_id', 'language']);
    $q->fields('e', ['language'], 'entity_language');
    $q->join($table, 'e', "f.entity_id = e.$id");
    $q->condition("e.$bundle", $this->bundle_name);

    if ($translation) {
      if (!$this->isTranslatable()) {
        return 'Not translatable';
      }

      if (!empty($info['translatable']) && $this->isTranslatable() == ENTITY_TRANSLATION_ENABLED) {
        $q->where('f.language != e.language');
      }
      elseif ($this->entity_name == 'node' && $this->isTranslatable() == TRANSLATION_ENABLED) {
        $q->where('e.nid != e.tnid AND e.tnid != 0');
        $q->join('node', 't', 't.nid = e.tnid');
        $q->join("field_data_$field_name", 'ft', 'ft.entity_id = t.nid');
        $q->where('ft.delta = f.delta');
        $q->where("ft.{$field_name}_{$column} != f.{$field_name}_{$column}");
      }
      else {
        return 'Not translatable';
      }
    }
    else {
      if ($this->isTranslatable()) {
        if (!empty($info['translatable']) && $this->isTranslatable() == ENTITY_TRANSLATION_ENABLED) {
          $q->where("f.language = e.language OR f.language = :lang", [':lang' => LANGUAGE_NONE]);
        }
        elseif ($this->entity_name == 'node' && $this->isTranslatable() == TRANSLATION_ENABLED) {
          $q->where('e.nid = e.tnid OR e.tnid = 0');
        }
      }
    }

    $result = $q->execute()->fetchAssoc();
    if (empty($result)) {
      return 'No results';
    }

    $language = $result['language'] != LANGUAGE_NONE ? $result['language'] : $result['e_language'];

    $uri_callback = $this->entity['uri callback'];
    $uri = call_user_func($uri_callback, reset(entity_load($this->entity_name, [$result['entity_id']])));

    $link = $uri['path'] . '/edit';
    $link_text = $link;
    if (language_default('language') != $language) {
      $link_text = language_list()[$language]->prefix . '/' . $link_text;
    }

    return l($link_text, $link, ['language' => language_list()[$language]]);
  }

}
